#include "Python.h"
#include "../header/ham-ndx.h"
#include "../header/keyword.h"
#include <ctype.h>

#define UNLESS(E)	if (!(E))


static PyObject *HAMLib_getKeywordList(PyObject *, PyObject *);


static char HAMLibModuleDocumentation[] = 
"Use HAMLib to parse source strings into sequence of words using HAM\n"
"\n"
"for use with the Catalog.\n"
;

static struct PyMethodDef HAMLibModuleMethods[] = {
    { "getKeywordList", (PyCFunction)HAMLib_getKeywordList, METH_VARARGS,
      "getKeywordList(text) -- Break the given string into words",
    },
    { NULL, NULL }		/* sentinel */
};


int substring(source, target, length)
unsigned char *source;
unsigned char *target;
unsigned int length; {
    int i;
    unsigned char ch;
    
    for (i = 0; i < (length - 1); i++) {
        ch = source[i];
        if (ch == '\0') break;
        target[i] = ch;
        if (ch & 0x80) {
            i++;
            target[i] = source[i];
        }
    }
    
    target[i] = '\0';
    
    return i;
}

static PyObject *HAMLib_getKeywordList(modinfo, args)
PyObject *modinfo;
PyObject *args; {
    unsigned char *source, *currPosition;
    unsigned char sentence[SENTSIZE];
    
    unsigned char *keywords[MAXKEYWORDS];
    unsigned char *keywordsMemory[MAXKEYBYTES];
    int noOfKeywords, charsRead, i;
    
    HAM_RUNMODE hamMode;
    HAM_MORES hamOut;
    
    PyObject *keywordList = Py_BuildValue("[]");
    PyObject *pyKeyword;
    
    UNLESS(keywordList) return NULL;
    UNLESS(PyArg_ParseTuple(args, "s", &source)) return NULL;
    
    if (open_HAM_index(&hamMode, NULL, "/usr/local/dc/Zope/hdic/ham2000.ini")) {
        fprintf(stderr, "Cannot find dictionary configuration file ham2000.ini\n");
        
        return NULL;
    }
    
    currPosition = source;
    while (charsRead = substring(currPosition, sentence, SENTSIZE)) {
    	noOfKeywords
          = get_stems_TS(sentence, keywords, keywordsMemory, &hamOut, &hamMode);
        
        for (i = 0; i < noOfKeywords; i++) {
            UNLESS(pyKeyword = PyString_FromString(keywords[i])) return NULL;
            UNLESS(! PyList_Append(keywordList, pyKeyword)) return NULL;
        }
        
        currPosition += charsRead;
    }
    
    close_HAM_index();
    
    return keywordList;
}

void initHAMLib() {
    PyObject *m, *d;
    
    /* Create the module and add the functions */
    m = Py_InitModule4("HAMLib", HAMLibModuleMethods,
                       HAMLibModuleDocumentation,
                       (PyObject *) NULL, PYTHON_API_VERSION);
    
    if (PyErr_Occurred()) Py_FatalError("can't initialize module HAMLib");
}
